@@ -2,17 +2,15 @@
 
 use crate::{Error, Result};
 use core::{
-    convert::{TryFrom, TryInto},
     fmt,
     ops::{Add, Range},
 };
-use der::{asn1::UIntBytes, Decodable};
+use der::{asn1::UIntBytes, Decodable, Encodable};
 use elliptic_curve::{
     bigint::Encoding as _,
     consts::U9,
     generic_array::{ArrayLength, GenericArray},
-    weierstrass::Curve,
-    FieldSize,
+    FieldSize, PrimeCurve,
 };
 
 #[cfg(feature = "alloc")]
@@ -45,7 +43,7 @@ type SignatureBytes<C> = GenericArray<u8, MaxSize<C>>;
 /// Generic over the scalar size of the elliptic curve.
 pub struct Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -61,7 +59,7 @@ where
 
 impl<C> signature::Signature for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -74,7 +72,7 @@ where
 #[allow(clippy::len_without_is_empty)]
 impl<C> Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -96,12 +94,21 @@ where
 
     /// Create an ASN.1 DER encoded signature from big endian `r` and `s` scalars
     pub(crate) fn from_scalar_bytes(r: &[u8], s: &[u8]) -> der::Result<Self> {
+        let r = UIntBytes::new(r)?;
+        let s = UIntBytes::new(s)?;
+
         let mut bytes = SignatureBytes::<C>::default();
         let mut encoder = der::Encoder::new(&mut bytes);
-        encoder.message(&[&UIntBytes::new(r)?, &UIntBytes::new(s)?])?;
 
-        let sig = encoder.finish()?;
-        sig.try_into().map_err(|_| der::Tag::Sequence.value_error())
+        encoder.sequence((r.encoded_len()? + s.encoded_len()?)?, |seq| {
+            seq.encode(&r)?;
+            seq.encode(&s)
+        })?;
+
+        encoder
+            .finish()?
+            .try_into()
+            .map_err(|_| der::Tag::Sequence.value_error())
     }
 
     /// Get the `r` component of the signature (leading zeros removed)
@@ -117,7 +124,7 @@ where
 
 impl<C> AsRef<[u8]> for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -128,7 +135,7 @@ where
 
 impl<C> fmt::Debug for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -142,7 +149,7 @@ where
 
 impl<C> TryFrom<&[u8]> for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -150,7 +157,11 @@ where
 
     fn try_from(input: &[u8]) -> Result<Self> {
         let (r, s) = der::Decoder::new(input)
-            .sequence(|decoder| Ok((UIntBytes::decode(decoder)?, UIntBytes::decode(decoder)?)))
+            .and_then(|mut decoder| {
+                decoder.sequence(|decoder| {
+                    Ok((UIntBytes::decode(decoder)?, UIntBytes::decode(decoder)?))
+                })
+            })
             .map_err(|_| Error::new())?;
 
         if r.as_bytes().len() > C::UInt::BYTE_SIZE || s.as_bytes().len() > C::UInt::BYTE_SIZE {
@@ -177,7 +188,7 @@ where
 
 impl<C> TryFrom<Signature<C>> for super::Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
@@ -207,7 +218,7 @@ fn find_scalar_range(outer: &[u8], inner: &[u8]) -> Result<Range<usize>> {
 #[cfg(all(feature = "digest", feature = "hazmat"))]
 impl<C> signature::PrehashSignature for Signature<C>
 where
-    C: Curve + crate::hazmat::DigestPrimitive,
+    C: PrimeCurve + crate::hazmat::DigestPrimitive,
     MaxSize<C>: ArrayLength<u8>,
     <FieldSize<C> as Add>::Output: Add<MaxOverhead> + ArrayLength<u8>,
 {
